PortalFu = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceEvent-2.0", "FuBarPlugin-2.0")
local PortalFu = PortalFu

local L = AceLibrary("AceLocale-2.2"):new("PortalFu")
local tablet = AceLibrary("Tablet-2.0")
local dewdrop = AceLibrary("Dewdrop-2.0")

local string_find = string.find
local string_sub = string.sub
local math_floor = math.floor

local GetSpellInfo = GetSpellInfo
local GetSpellName = GetSpellName
local GetContainerNumSlots = GetContainerNumSlots
local GetContainerItemInfo = GetContainerItemInfo
local GetContainerItemLink = GetContainerItemLink
local GetContainerItemCooldown = GetContainerItemCooldown
local GetBindLocation = GetBindLocation

local scrollOfRecallIcon = "Interface\\Icons\\INV_Scroll_16"
local hearthstoneIcon	= "Interface\\Icons\\INV_Misc_Rune_01"

PortalFu.hasIcon = true
PortalFu.canHideText = true
PortalFu.hasNoColor = true
PortalFu.defaultMinimapPosition = 20

PortalFu:RegisterDB("PortalFuDB", "PortalFuPerCharDB")
PortalFu:RegisterDefaults('profile', {
	showTag = true,
})


local function pairsByKeys(t)
	local a = {}
	for n in pairs(t) do
		table.insert(a, n)
	end
	table.sort(a)
	
	local i = 0
	local iter = function ()
		i = i + 1
		if a[i] == nil then
			return nil
		else
			return a[i], t[a[i]]
		end
	end
	return iter
end

local function findSpell(spellName)
	local i = 1
	while true do
		local s = GetSpellName(i, BOOKTYPE_SPELL)
		if not s then
			break
		end
		
		if s == spellName then
			return i
		end
		
		i = i + 1
	end
end

local function hasItem(itemName)
	for bag = 0, 4 do
    for slot = 1, GetContainerNumSlots(bag) do
      local item = GetContainerItemLink(bag, slot)
      if item then
        if string_find(item, itemName) then
          return true
        end
      end
    end
  end
	
	return false
end

local function getHearthCooldown()
	local cooldown, startTime, duration

  for bag = 0, 4 do
    for slot = 1, GetContainerNumSlots(bag) do
      local item = GetContainerItemLink(bag, slot)
      if item then
        if string_find(item, L["HEARTHSTONE"]) then
          startTime, duration = GetContainerItemCooldown(bag, slot)
          cooldown = duration - (GetTime() - startTime)
          cooldown = cooldown / 60
          cooldown = math_floor(cooldown)
          if cooldown <= 0 then
            return L["READY"]
          end

          return cooldown.." "..L["MIN"]
        end
      end
    end
  end
  
  return L["N/A"]
end

local function getReagentCount(name)
	local count = 0
  for bag = 0, 4 do
    for slot = 1, GetContainerNumSlots(bag) do
      local item = GetContainerItemLink(bag, slot)
      if item then
        if string_find(item, name) then
          local _, itemCount = GetContainerItemInfo(bag, slot)
          count = count + itemCount
        end
      end
    end
  end
	
	return count
end

function PortalFu:OnInitialize()

	self:RegisterChatCommand("/portalfu", function() PortalFu:ToggleMinimapAttached() end)
	
	local mageSpells = {
		Alliance = {
			3561,  --TP:Stormwind
			3562,  --TP:Ironforge
			3565,  --TP:Darnassus
			32271, --TP:Exodar
			49359, --TP:Theramore
			33690, --TP:Shattrath
			53140, --TP:Dalaran
			10059, --P:Stormwind
			11416, --P:Ironforge
			11419, --P:Darnassus
			32266, --P:Exodar
			49360, --P:Theramore
			33691, --P:Shattrath
			53142  --P:Dalaran
		},
		Horde = {
			3563,  --TP:Undercity
			3566,  --TP:Thunder Bluff
			3567,  --TP:Orgrimmar
			32272, --TP:Silvermoon
			49358, --TP:Stonard
			35715, --TP:Shattrath
			53140, --TP:Dalaran
			11418, --P:Undercity
			11420, --P:Thunder Bluff
			11417, --P:Orgrimmar
			32267, --P:Silvermoon
			49361, --P:Stonard
			35717, --P:Shattrath
			53142  --P:Dalaran
		}
	}
	
	local _, class = UnitClass("player")
	if class == "MAGE" then
		self.portals = mageSpells[UnitFactionGroup("player")]
	elseif class == "DEATHKNIGHT" then
		self.portals = {
			50977 --Death Gate
			}
	elseif class == "DRUID" then
		self.portals = {
			18960 --TP:Moonglade
			}
	elseif class == "SHAMAN" then
		self.portals = {
			556 --Astral Recall
			}
	end

	spells = nil
end

function PortalFu:OnEnable()
	self.METHODS = {}
	self.lastCast = L["N/A"]
	self:UpdateSpells()
	self:RegisterEvent("SKILL_LINES_CHANGED")
	self:UpdateDisplay()
	self:SetIcon(hearthstoneIcon)
end

function PortalFu:OnDisable()
	self.METHODS = nil
	self.lastCast = nil
	self:UnregisterAllEvents()
end

function PortalFu:SKILL_LINES_CHANGED()
	self:UpdateSpells()
end

function PortalFu:OnMenuRequest(level,value)
	if level == 1 then
		for k,v in pairsByKeys(self.METHODS) do
			if v.secure then
				dewdrop:AddLine(
					'text', v.text,
					'secure', v.secure,
					'icon', v.spellIcon,
					'func', function() 
						self:SetIcon(v.spellIcon)
						self.lastCast = v.text
						self:UpdateDisplay()
					end,
					'disabled', false,
					'closeWhenClicked', true
				)
			end
		end
		dewdrop:AddLine()
		
		self:ShowHearthstone()
	end
end

function PortalFu:UpdateText()
	if self.lastCast then
		local dots = string_find(self.lastCast, ":")
		if dots ~= nil then
			self.lastCast = string_sub(self.lastCast, dots + 2)
		end
		self:SetText(self.lastCast)
	else
		self:SetText(L["N/A"])
	end
end

function PortalFu:OnTooltipUpdate()
	local cat = tablet:AddCategory(
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_text2R', 1,
		'child_text2G', 1,
		'child_text2B', 1
	)
	cat:AddLine('text', L["RCLICK"], 'text2', L["SEE_SPELLS"])
	cat:AddLine('text', ' ')
	cat:AddLine('text', L["HEARTHSTONE"].." : "..GetBindLocation(), 'text2', getHearthCooldown())
	cat:AddLine('text', ' ')
	cat:AddLine('text', L["TP_P"], 'text2', getReagentCount(L["TP_RUNE"]).."/"..getReagentCount(L["P_RUNE"]))
end

function PortalFu:UpdateSpells()
	if self.portals then
		for _,unTransSpell in ipairs(self.portals) do
			
			local spell, _, spellIcon = GetSpellInfo(unTransSpell)
			local spellid = findSpell(spell)
			
			if spellid then	
				self.METHODS[spell] = {
					spellid = spellid,
					text = spell,
					spellIcon = spellIcon,
					secure = {
						type = 'spell',
						spell = spell,
					}
				}
			end
		end
	end
end

function PortalFu:ShowHearthstone()
	local text, secure, icon
	
	local hsCd = getHearthCooldown()
	if hsCd == L["READY"] then
		local bindLoc = GetBindLocation()
		if bindLoc then
			text = L["INN"].." "..bindLoc
			secure = {
				type = 'item',
				item = L["HEARTHSTONE"],
			}
			icon = hearthstoneIcon
		end
	else
		if hasItem(L["SCROLL_3"]) then
			text = L["SCROLL_3"]
			secure = {
				type = 'item',
				item = L["SCROLL_3"],
			}
			icon = scrollOfRecallIcon
		end
	end
	
	if secure ~= nil then
		dewdrop:AddLine(
			'text', text,
			'secure', secure,
			'icon', icon,
			'func', function() 
				self:SetIcon(icon)
				self.lastCast = text
				self:UpdateDisplay()
			end,
			'closeWhenClicked', true
		)
		dewdrop:AddLine()
	end
end
